from urllib.parse import urljoin
from requests import Session, Response

from .singleton import SingletonMeta
from .exceptions import BadTokenException


AUTH_HEADER = "x-access-token"


class Client(metaclass=SingletonMeta):
    __token: str = None
    __refresh_token: str = None
    __url: str = None
    __session = Session()

    @classmethod
    def set_token(cls, token: str):
        cls.__refresh_token = token

    @classmethod
    def set_url(cls, url: str):
        cls.__url = url

    @classmethod
    def _refresh_token(cls):
        url = urljoin(cls.__url, 'internal/auth/token/refresh')
        response = cls.__session.post(url, params={"refresh_token": cls.__refresh_token})
        if response.status_code == 401:
            raise BadTokenException(response.json()["detail"])
        cls.__token = response.text[1:-1]

    def request(self, method: str, service_path_prefix: str, path, query, body, headers, cookies, **kwargs) -> Response:
        if self.__class__.__url is None:
            raise RuntimeError(
                "Can't use uninitialized client set url and token first "
                "(Client.set_url('...'), Client.set_token('...'))"
            )
        if self.__class__.__token is None:
            self.__class__._refresh_token()

        url = urljoin(self.__url, service_path_prefix)
        url = url + path

        headers.update({"Content-Type": kwargs.pop("content_type", "application/json")})
        headers.update({AUTH_HEADER: self.__token})
        
        response = self.__session.request(method, url, params=query, body=body, headers=headers, cookies=cookies, **kwargs)
        if response.headers.get("x-auth-exception", None) == "Expired":
            self.__class__._refresh_token()
            headers.update({AUTH_HEADER: self.__token})
            response = self.__session.request(method, url, params=query, body=body, headers=headers, cookies=cookies, **kwargs)
        elif response.headers.get("x-auth-exception", None) == "Invalid":
            raise BadTokenException("invalid token")
        elif response.headers.get("x-auth-exception", None) == "Not Authorized":
            required_permission = response.headers.get("x-required-scope", None)
            raise BadTokenException(
                f"you do not have to permissions to make that request (required permission: {required_permission})"
            )
        return response



import setuptools

with open("README.md", "r") as f:
    long_description = f.read()


setuptools.setup(
    name='connectapi-core',
    packages=setuptools.find_packages(),
    version='1.1.3',
    license='MIT',
    description="""Core SDK for the ConnectAPI ecosystem""",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='ConnectAPI',
    author_email='connectapi.org@gmail.com',
    maintainer='ConnectAPI',
    maintainer_email='connectapi.org@gmail.com',
    url='https://github.com/ConnectAPI/python-core-client',
    keywords=['connectapi', 'ConnectAPI', 'web', 'client', 'core'],
    install_requires=['requests'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Topic :: Software Development',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
)



class SingletonMeta(type):
    _instance = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instance:
            cls._instance[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
        return cls._instance[cls]

